@@ -1,30 +1,72 @@
-from __future__ import unicode_literals
+import uuid
 
 from django.db.models import Sum
 from django.utils.translation import ugettext_lazy as _
 
-from .helpers import get_calculation_annotation
 from .decorators import report_field_register
+from .helpers import get_calculation_annotation
 from .registry import field_registry
 
 
 class BaseReportField(object):
-    # date_field = 'doc_date'
-    plus_minus_modifier_field = 'doc_type'
+    """
+    Computation field responsible for making the calculation unit
+    """
+
+    _field_registry = field_registry
+    name = ''
+    """The name to be used in the ReportGenerator"""
+
     calculation_field = 'value'
+    """the Field to compute on"""
+
     calculation_method = Sum
+    """The computation Method"""
+
+    verbose_name = None
+    """Verbose name to be used in front end when needed"""
+
+    requires = None
+    """This can be a list of sibling classes,
+    they will be asked to compute and their value would be available to you in the `resolve` method
+    requires = [BasicCalculationA, BasicCalculationB]
+    """
+
+    type = 'number'
+    """Just a string describing what this computation field return"""
+
     report_model = None
+    group_by = None
     plus_side_q = None
     minus_side_q = None
-    _debit_and_credit = True
-    component_of = None
-    name = None
-    verbose_name = None
-    type = 'date'
-    requires = None
-    _require_classes = None
 
-    group_by = None
+    _require_classes = None
+    _debit_and_credit = True
+
+    @classmethod
+    def create(cls, method, field, name=None, verbose_name=None):
+        """
+        Creates a ReportField class on the fly
+        :param method:
+        :param field:
+        :param name:
+        :param verbose_name:
+        :return:
+        """
+        if not name:
+            identifier = str(uuid.uuid4()).split('-')[-1]
+            name = name or f"__{method.name}_{field}_{identifier}__"
+            assert name not in cls._field_registry.get_all_report_fields_names()
+
+        verbose_name = verbose_name or f'{method.name} {field}'
+        report_klass = type(f'ReportField_{name}', (BaseReportField,), {
+            'name': name,
+            'verbose_name': verbose_name,
+            'calculation_field': field,
+            'calculation_method': method
+        })
+        cls._field_registry.register(report_klass)
+        return name
 
     def __init__(self, plus_side_q=None, minus_side_q=None,
                  report_model=None,
@@ -65,6 +107,17 @@ class BaseReportField(object):
         return queryset
 
     def prepare(self, q_filters=None, kwargs_filters=None, **kwargs):
+        """
+        This is the first hook where you can customize the calculation away from the Django Query aggregation method
+        This method et called with all available parameters , so you can prepare the results for the whole set and save
+        it in a local cache (like self._cache) .
+        The flow will later call the method `resolve`,  giving you the id, for you to return it respective calculation
+
+        :param q_filters:
+        :param kwargs_filters:
+        :param kwargs:
+        :return:
+        """
         kwargs_filters = kwargs_filters or {}
 
         dep_values = self._prepare_dependencies(q_filters, kwargs_filters.copy())
@@ -128,6 +181,15 @@ class BaseReportField(object):
         return self.final_calculation(debit_value, credit_value, dependencies_value)
 
     def get_dependency_value(self, current_obj, name=None):
+        """
+        Get the values of the ReportFields specified in `requires`
+
+        :param current_obj: the current object which we want the calculation for
+        :param name: Optional, the name of the specific dependency you want.
+
+        :return: a dict containing dependencies names as keys and their calculation as values
+                 or a specific value if name is specified.
+        """
         values = self._resolve_dependencies(current_obj)
         if name:
             return values.get(name)
@@ -205,6 +267,27 @@ class BaseReportField(object):
 
         return get_dependency(cls)
 
+    @classmethod
+    def get_crosstab_field_verbose_name(cls, model, id):
+        """
+        Construct a verbose name for the crosstab field
+        :param model: the model name
+        :param id: the id of the current crosstab object
+        :return: a verbose string
+        """
+        return f'{cls.verbose_name} {model} {id}'
+
+    @classmethod
+    def get_time_series_field_verbose_name(cls, date_period):
+        """
+        Sent the column data to construct a verbose name.
+        Default implemenetation is column name + the end date %Y%m%d
+        :param column_name: the computation field_name
+        :param date_period: a tuple of (start_date, end_date)
+        :return: a verbose string
+        """
+        return f'{cls.verbose_name} {date_period[1].strftime("%Y%m%d")}'
+
 
 class FirstBalanceField(BaseReportField):
     name = '__fb__'
