@@ -14,33 +14,102 @@ logger = logging.getLogger(__name__)
 
 
 class ReportGenerator(object):
-    field_registry_class = field_registry
     """
-    Class to generate a Json Object containing report data based on
-    report form , main_queryset and teh report model
+    The main class responsible generating the report and managing the flow
+    """
+
+    field_registry_class = field_registry
+    """You can have a custom computation field locator! It only needs a `get_field_by_name(string)` 
+    and returns a ReportField`"""
+
+    report_model = None
+    """The main model where data is """
+
+    """
+    Class to generate a Json Object containing report data.
     """
     date_field = None
+    """Main date field to use whenever date filter is needed"""
+
     print_flag = None
     list_display_links = []
 
-    # V2
     group_by = None
+    """The field to use for grouping, if not set then the report is expected to be a sub version of the report model"""
+
     columns = None
+    """A list of column names.
+    Columns names can be 
+    
+    1. A Computation Field
+    
+    2. If group_by is set, then any field on teh group_by model
+    
+    3. If group_by is not set, then any field name on the report_model / queryset
+     
+    4. A callable on the generator
+      
+    5. Special __time_series__, and __crosstab__ 
+       Those can be use to control the position of the time series inside the columns, defaults it's appended at the end
+       
+       Example:
+       columns = ['product_id', '__time_series__', 'col_b']
+       Same is true with __crosstab__ 
+     """
 
     time_series_pattern = ''
+    """
+    If set the Report will compute a time series.
+    
+    Possible options are: daily, weekly, semimonthly, monthly, quarterly, semiannually, annually and custom.
+    
+    if `custom` is set, you'd need to override  `get_custom_time_series_dates`
+    """
     time_series_columns = None
-    show_empty_records = None
-    report_model = None
+    """
+    a list of Calculation Field names which will be included in the series calculation.
+     Example: ['__total__', '__total_quantity__'] with compute those 2 fields for all the series
+     
+    """
+
     crosstab_model = None
+    """
+    If set, a cross tab over this model selected ids (via `crosstab_ids`)  
+    """
     crosstab_columns = None
+    """The computation fields which will be computed for each crosstab-ed ids """
+
+    crosstab_ids = None
+    """A list is the ids to create a crosstab report on"""
+
+    crosstab_compute_reminder = True
+    """Include an an extra crosstab_columns for the outer group ( ie: all expects those `crosstab_ids`) """
+
+    show_empty_records = True
+    """
+    If group_by is set, this option control if the report result will include all objects regardless of appearing in the report_model/qs.
+    If set False, only those objects which are found in the report_model/qs
+    Example: Say you group by client
+    show_empty_records = True will get the computation fields for all clients in the Client model (including those who 
+    didnt make a transaction.
+    
+    show_empty_records = False will get the computation fields for all clients in the Client model (including those who 
+    didnt make a transaction.
+     
+    """
+
     limit_records = None
+    """Serves are a main limit to  the returned data of teh report_model.
+    Can be beneficial if the results may be huge.
+    """
     swap_sign = False
 
     def __init__(self, report_model=None, main_queryset=None, start_date=None, end_date=None, date_field=None,
                  q_filters=None, kwargs_filters=None,
-                 group_by=None, columns=None, time_series_pattern=None, time_series_columns=None,
-                 crosstab_model=None, crosstab_columns=None, crosstab_ids=None, crosstab_compute_reminder=True,
-                 swap_sign=False, show_empty_records=True,
+                 group_by=None, columns=None,
+                 time_series_pattern=None, time_series_columns=None,
+                 crosstab_model=None, crosstab_columns=None, crosstab_ids=None, crosstab_compute_reminder=None,
+                 swap_sign=False, show_empty_records=None,
                  print_flag=False,
                  doc_type_plus_list=None, doc_type_minus_list=None, limit_records=False, ):
         """
@@ -88,14 +157,14 @@ class ReportGenerator(object):
         self.q_filters = q_filters or []
         self.kwargs_filters = kwargs_filters or {}
 
-        self.crosstab_model = crosstab_model or self.crosstab_model
-        self.crosstab_columns = crosstab_columns or self.crosstab_columns
-        self.crosstab_ids = crosstab_ids or []
-        self.crosstab_compute_reminder = crosstab_compute_reminder
+        self.crosstab_model = self.crosstab_model or crosstab_model
+        self.crosstab_columns = crosstab_columns or self.crosstab_columns or []
+        self.crosstab_ids = self.crosstab_ids or crosstab_ids or []
+        self.crosstab_compute_reminder = self.crosstab_compute_reminder if crosstab_compute_reminder is None else crosstab_compute_reminder
 
         main_queryset = main_queryset or self.report_model.objects
 
-        self.columns = self.columns or columns
+        self.columns = self.columns or columns or []
         self.group_by = self.group_by or group_by
 
         self.time_series_pattern = self.time_series_pattern or time_series_pattern
@@ -108,7 +177,8 @@ class ReportGenerator(object):
         self.existing_dependencies = {'series': [], 'matrix': [], 'normal': []}
 
         self.print_flag = print_flag or self.print_flag
-        #
+
+        # todo validate columns is not empty (if no time series / cross tab)
 
         if self.group_by:
             try:
@@ -136,22 +206,31 @@ class ReportGenerator(object):
 
         # in case of a group by, do we show a grouped by model data regardless of their appearance in the results
         # a client who didnt make a transaction during the date period.
-        self.show_empty_records = self.show_empty_records or show_empty_records
+        self.show_empty_records = False # show_empty_records if show_empty_records else self.show_empty_records
+        # Looks like this options is harder then what i thought as it interfere with the usual filtering of the report
 
         # Preparing actions
         self._parse()
         if self.group_by:
+
             if self.show_empty_records:
-                self.main_queryset = self.group_by_field.related_model.objects.values()
+                pass
+                # group_by_filter = self.kwargs_filters.get(self.group_by, '')
+                # qs = self.group_by_field.related_model.objects
+                # if group_by_filter:
+                #     import pdb; pdb.set_trace()
+                #     lookup = 'pk__in' if isinstance(group_by_filter, Iterable) else 'pk'
+                #     qs = qs.filter(**{lookup: group_by_filter})
+                # self.main_queryset = qs.values()
+
             else:
                 self.main_queryset = self._apply_queryset_options(main_queryset)
-                ids = main_queryset.values_list(self.group_by_field.attname)
+                ids = self.main_queryset.values_list(self.group_by_field.attname).distinct()
                 self.main_queryset = self.group_by_field.related_model.objects.filter(pk__in=ids).values()
         else:
             self.main_queryset = self._apply_queryset_options(main_queryset, self.get_database_columns())
 
         self._prepare_report_dependencies()
-        # self.prepare_calculation()
 
     def _apply_queryset_options(self, query, fields=None):
         """
@@ -256,7 +335,7 @@ class ReportGenerator(object):
 
                 name = col_data['name']
 
-                if name.startswith('__') and self.group_by:
+                if col_data.get('source', '') == 'magic_field' and self.group_by:
                     source = self._report_fields_dependencies[window].get(name, False)
                     if source:
                         computation_class = self.report_fields_classes[source]
@@ -291,7 +370,6 @@ class ReportGenerator(object):
         return data
 
     def _parse(self):
-        from .registry import field_registry
 
         if self.group_by:
             self.group_by_field = [x for x in self.report_model._meta.fields if x.name == self.group_by][0]
@@ -330,8 +408,6 @@ class ReportGenerator(object):
                     else:
                         field = model_to_use._meta.get_field(col)
                 except FieldDoesNotExist:
-                    import pdb;
-                    pdb.set_trace()
                     raise FieldDoesNotExist(
                         f'Field "{col}" not found as an attribute to the generator class, nor as computation field, nor as a database column for the model "{model_to_use._meta.model_name}"')
 
@@ -394,21 +470,26 @@ class ReportGenerator(object):
 
                 _values.append({
                     'name': col + 'TS' + dt[1].strftime('%Y%m%d'),
-                    'verbose_name': self.get_time_series_field_verbose_name(col, dt),
+                    'original_name': col,
+                    'verbose_name': self.get_time_series_field_verbose_name(magic_field_class, dt),
                     'ref': magic_field_class,
                     'start_date': dt[0],
                     'end_date': dt[1],
+                    'source': 'magic_field' if magic_field_class else '',
                 })
         return _values
 
-    def get_time_series_field_verbose_name(self, column_name, date_period):
+    def get_time_series_field_verbose_name(self, computation_class, date_period):
         """
-        Sent the column data to construct a verbose name, below is only a basic implementation
-        :param column_name:
-        :param date_period:
-        :return:
+        Sent the column data to construct a verbose name.
+        Default implementation is delegated to the ReportField.get_time_series_field_verbose_name
+        (which is  name + the end date %Y%m%d)
+
+        :param computation_class: the computation field_name
+        :param date_period: a tuple of (start_date, end_date)
+        :return: a verbose string
         """
-        return column_name + date_period[1].strftime('%Y%m%d')
+        return computation_class.get_time_series_field_verbose_name(date_period)
 
     def get_custom_time_series_dates(self):
         """
@@ -473,24 +554,22 @@ class ReportGenerator(object):
 
                 output_cols.append({
                     'name': f'{col}CT{id}',
-                    'verbose_name': self.get_crosstab_field_verbose_name(col, id),
+                    'verbose_name': self.get_crosstab_field_verbose_name(magic_field_class, self.crosstab_model, id),
                     'ref': magic_field_class,
                     'id': id,
                     'model': self.crosstab_model,
-                    'is_reminder': counter == ids_length
+                    'is_reminder': counter == ids_length,
+                    'source': 'magic_field' if magic_field_class else '',
                 })
 
         return output_cols
 
-        # entity_name = self.crosstab_model
-        # ids = [entity_name + '-' + id for id in ids if id != '']
-        #
-        # report_columns = [col + 'MX' for col in report_columns]
-        #
-        # _values = []
-        # if ids:
-        #     _values = [col + dt for dt in ids for col in report_columns]
-        # return _values
-
-    def get_crosstab_field_verbose_name(self, col, id):
-        return f'{col}CT{id}'
+    def get_crosstab_field_verbose_name(self, computation_class, model, id):
+        """
+        Hook to change the crosstab field verbose name, default it delegate this function to the ReportField
+        :param computation_class: ReportField Class
+        :param model: the model name as string
+        :param id: the current crosstab id
+        :return: a verbose string
+        """
+        return computation_class.get_crosstab_field_verbose_name(model, id)
